@@ -11,7 +11,8 @@ from ..base_handler import APIBase
 from ..auth import authorization, attr_auth
 from iceprod.core import dataclasses
 from iceprod.core.resources import Resources
-from iceprod.server.util import nowstr, task_statuses, task_status_sort
+from iceprod.server.util import nowstr
+from iceprod.server.states import TASK_STATUS, TASK_STATUS_START, task_status_sort, task_prev_statuses
 
 logger = logging.getLogger('rest.tasks')
 
@@ -31,11 +32,13 @@ def setup(handler_cfg):
             (r'/tasks', MultiTasksHandler, handler_cfg),
             (r'/tasks/(?P<task_id>\w+)', TasksHandler, handler_cfg),
             (r'/tasks/(?P<task_id>\w+)/status', TasksStatusHandler, handler_cfg),
-            (r'/task_actions/queue', TasksActionsQueueHandler, handler_cfg),
             (r'/task_actions/bulk_status/(?P<status>\w+)', TaskBulkStatusHandler, handler_cfg),
-            (r'/task_actions/process', TasksActionsProcessingHandler, handler_cfg),
+            (r'/task_actions/waiting', TasksActionsWaitingHandler, handler_cfg),
+            (r'/task_actions/queue', TasksActionsQueueHandler, handler_cfg),
             (r'/task_counts/status', TaskCountsStatusHandler, handler_cfg),
+            (r'/tasks/(?P<task_id>\w+)/task_actions/processing', TasksActionsProcessingHandler, handler_cfg),
             (r'/tasks/(?P<task_id>\w+)/task_actions/reset', TasksActionsErrorHandler, handler_cfg),
+            (r'/tasks/(?P<task_id>\w+)/task_actions/failed', TasksActionsFailedHandler, handler_cfg),
             (r'/tasks/(?P<task_id>\w+)/task_actions/complete', TasksActionsCompleteHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/tasks', DatasetMultiTasksHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/tasks/(?P<task_id>\w+)', DatasetTasksHandler, handler_cfg),
@@ -44,7 +47,10 @@ def setup(handler_cfg):
             (r'/datasets/(?P<dataset_id>\w+)/task_counts/status', DatasetTaskCountsStatusHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/task_counts/name_status', DatasetTaskCountsNameStatusHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_status/(?P<status>\w+)', DatasetTaskBulkStatusHandler, handler_cfg),
-            (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_requirements/(?P<name>\w+)', DatasetTaskBulkRequirementsHandler, handler_cfg),
+            (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_suspend', DatasetTaskBulkSuspendHandler, handler_cfg),
+            (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_reset', DatasetTaskBulkResetHandler, handler_cfg),
+            (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_hard_reset', DatasetTaskBulkHardResetHandler, handler_cfg),
+            (r'/datasets/(?P<dataset_id>\w+)/task_actions/bulk_requirements/(?P<name>[^\/\?\#]+)', DatasetTaskBulkRequirementsHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/task_stats', DatasetTaskStatsHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/files', DatasetMultiFilesHandler, handler_cfg),
             (r'/datasets/(?P<dataset_id>\w+)/files/(?P<task_id>\w+)', DatasetTaskFilesHandler, handler_cfg),
@@ -147,6 +153,24 @@ class MultiTasksHandler(APIBase):
                 r = 'key {} should be of type {}'.format(k, req_fields[k])
                 raise tornado.web.HTTPError(400, reason=r)
 
+        opt_fields = {
+            'status': str,
+            'priority': float,
+            'instance_id': str,
+        }
+        for k in opt_fields:
+            if k in data and not isinstance(data[k], opt_fields[k]):
+                r = 'key "{}" should be of type {}'.format(k, opt_fields[k].__name__)
+                raise tornado.web.HTTPError(400, reason=r)
+
+        bad_fields = set(data).difference(set(opt_fields).union(req_fields))
+        if bad_fields:
+            r = 'invalid keys found'
+            raise tornado.web.HTTPError(400, reason=r)
+
+        if 'status' in data and data['status'] not in TASK_STATUS:
+            raise tornado.web.HTTPError(400, reason='invalid status')
+
         # set some fields
         task_id = uuid.uuid1().hex
         data.update({
@@ -160,9 +184,11 @@ class MultiTasksHandler(APIBase):
             'site': '',
         })
         if 'status' not in data:
-            data['status'] = 'waiting'
+            data['status'] = TASK_STATUS_START
         if 'priority' not in data:
             data['priority'] = 1.
+        if 'instance_id' not in data:
+            data['instance_id'] = ''
 
         await self.db.tasks.insert_one(data)
         self.set_status(201)
@@ -232,7 +258,9 @@ class TasksStatusHandler(APIBase):
         """
         Set a task status.
 
-        Body should have {'status': <new_status>}
+        Body args:
+            status (str): new status
+            instance_id (str): matching instance id (optional)
 
         Args:
             task_id (str): the task id
@@ -243,19 +271,39 @@ class TasksStatusHandler(APIBase):
         data = json.loads(self.request.body)
         if (not data) or 'status' not in data:
             raise tornado.web.HTTPError(400, reason='Missing status in body')
-        if data['status'] not in task_statuses:
+        status = data['status']
+        if status not in TASK_STATUS:
             raise tornado.web.HTTPError(400, reason='Bad status')
+
+        search = {
+            'task_id': task_id,
+            'status': {'$in': task_prev_statuses(status)},
+        }
+
         update_data = {
-            'status': data['status'],
+            'status': status,
             'status_changed': nowstr(),
         }
 
-        ret = await self.db.tasks.update_one({'task_id':task_id}, {'$set':update_data})
+        instance_id = data.get('instance_id', '')
+        if instance_id:
+            search['instance_id'] = instance_id
+
+        ret = await self.db.tasks.update_one(
+            search,
+            {'$set': update_data}
+        )
         if (not ret) or ret.modified_count < 1:
-            self.send_error(404, reason="Task not found")
-        else:
-            self.write({})
-            self.finish()
+            ret = await self.db.tasks.find_one({'task_id': task_id})
+            if not ret:
+                self.send_error(404, reason="Task not found")
+                return
+            elif ret['status'] != status:
+                self.send_error(400, reason="Bad state transition for status")
+                return
+
+        self.write({})
+        self.finish()
 
 
 class TaskCountsStatusHandler(APIBase):
@@ -271,8 +319,8 @@ class TaskCountsStatusHandler(APIBase):
             dict: {<status>: num}
         """
         ret = {}
-        for status in task_statuses:
-            ret[status] = await self.db.tasks.count_documents({"status":status})
+        for status in TASK_STATUS:
+            ret[status] = await self.db.tasks.count_documents({"status": status})
 
         ret2 = {}
         for k in sorted(ret, key=task_status_sort):
@@ -307,7 +355,10 @@ class DatasetMultiTasksHandler(APIBase):
 
         status = self.get_argument('status', None)
         if status:
-            filters['status'] = {'$in': status.split('|')}
+            status_list = status.split('|')
+            if any(s not in TASK_STATUS for s in status_list):
+                raise tornado.web.HTTPError(400, reaosn='Unknown task status')
+            filters['status'] = {'$in': status_list}
 
         job_id = self.get_argument('job_id', None)
         if job_id:
@@ -368,6 +419,55 @@ class DatasetTasksHandler(APIBase):
 
 
 class DatasetTasksStatusHandler(APIBase):
+    """
+    Handle single task requests.
+    """
+    @authorization(roles=['admin', 'user', 'system'])
+    @attr_auth(arg='dataset_id', role='write')
+    async def put(self, dataset_id, task_id):
+        """
+        Set a task status, following possible state transitions.
+
+        Body should have {'status': <new_status>}
+
+        Args:
+            dataset_id (str): dataset id
+            task_id (str): the task id
+
+        Returns:
+            dict: empty dict
+        """
+        data = json.loads(self.request.body)
+        if (not data) or 'status' not in data:
+            raise tornado.web.HTTPError(400, reason='Missing status in body')
+        if data['status'] not in TASK_STATUS:
+            raise tornado.web.HTTPError(400, reason='Bad status')
+        update_data = {
+            'status': data['status'],
+            'status_changed': nowstr(),
+            'instance_id': '',
+        }
+        if data['status'] == 'reset':
+            update_data['failures'] = 0
+
+        ret = await self.db.tasks.update_one(
+            {'task_id': task_id, 'dataset_id': dataset_id, 'status': {'$in': task_prev_statuses(data['status'])}},
+            {'$set': update_data}
+        )
+        if (not ret) or ret.modified_count < 1:
+            ret = await self.db.tasks.find_one({'task_id': task_id, 'dataset_id': dataset_id})
+            if not ret:
+                self.send_error(404, reason="Task not found")
+                return
+            elif ret['status'] != data['status']:
+                self.send_error(400, reason="Bad state transition for status")
+                return
+
+        self.write({})
+        self.finish()
+
+
+class DatasetTasksForceStatusHandler(APIBase):
     """
     Handle single task requests.
     """
@@ -389,11 +489,12 @@ class DatasetTasksStatusHandler(APIBase):
         data = json.loads(self.request.body)
         if (not data) or 'status' not in data:
             raise tornado.web.HTTPError(400, reason='Missing status in body')
-        if data['status'] not in task_statuses:
+        if data['status'] not in TASK_STATUS:
             raise tornado.web.HTTPError(400, reason='Bad status')
         update_data = {
             'status': data['status'],
             'status_changed': nowstr(),
+            'instance_id': '',
         }
         if data['status'] == 'reset':
             update_data['failures'] = 0
@@ -452,8 +553,8 @@ class DatasetTaskCountsStatusHandler(APIBase):
             dict: {<status>: num}
         """
         cursor = self.db.tasks.aggregate([
-            {'$match':{'dataset_id':dataset_id}},
-            {'$group':{'_id':'$status', 'total': {'$sum':1}}},
+            {'$match': {'dataset_id': dataset_id}},
+            {'$group': {'_id': '$status', 'total': {'$sum': 1}}},
         ])
         ret = {}
         async for row in cursor:
@@ -482,11 +583,11 @@ class DatasetTaskCountsNameStatusHandler(APIBase):
             dict: {<name>: {<status>: num}}
         """
         cursor = self.db.tasks.aggregate([
-            {'$match':{'dataset_id':dataset_id}},
+            {'$match':{'dataset_id': dataset_id}},
             {'$group':{
-                '_id':{'name':'$name','status':'$status'},
-                'ordering':{'$first':'$task_index'},
-                'total': {'$sum':1}
+                '_id':{'name': '$name', 'status': '$status'},
+                'ordering': {'$first': '$task_index'},
+                'total': {'$sum': 1}
             }},
         ])
         ret = defaultdict(dict)
@@ -547,6 +648,38 @@ class DatasetTaskStatsHandler(APIBase):
         self.finish()
 
 
+class TasksActionsWaitingHandler(APIBase):
+    """
+    Handle task action for idle -> waiting.
+    """
+    @authorization(roles=['admin', 'system'])
+    async def post(self):
+        """
+        Take a list of task_ids and move them to waiting.
+
+        Body args (json):
+            task_ids: list
+
+        Returns:
+            dict: {waiting: num tasks waiting}
+        """
+        data = json.loads(self.request.body)
+        task_ids = data.get('task_ids', [])
+        if len(task_ids) > 100:
+            raise tornado.web.HTTPError(400, reason='too many tasks. must be <= 100')
+
+        query = {
+            'status': 'idle',
+            'task_id': {'$in': task_ids},
+        }
+        val = {'$set': {'status': 'waiting'}}
+
+        ret = await self.db.tasks.update_many(query, val)
+        waiting = ret.modified_count
+        logger.info(f'waiting {waiting} tasks')
+        self.write({'waiting': waiting})
+
+
 class TasksActionsQueueHandler(APIBase):
     """
     Handle task action for waiting -> queued.
@@ -554,46 +687,7 @@ class TasksActionsQueueHandler(APIBase):
     @authorization(roles=['admin', 'system'])
     async def post(self):
         """
-        Take a number of waiting tasks and queue them.
-
-        Order by priority.
-
-        Body args (json):
-            num_tasks: int
-
-        Returns:
-            dict: {queued: num tasks queued}
-        """
-        data = json.loads(self.request.body)
-        num_tasks = data.get('num_tasks', 100)
-
-        query = {'status': 'waiting'}
-        val = {'$set': {'status': 'queued'}}
-
-        queued = 0
-        while queued < num_tasks:
-            ret = await self.db.tasks.find_one_and_update(
-                query,
-                val,
-                projection={'_id':False,'task_id':True},
-                sort=[('priority', -1)]
-            )
-            if not ret:
-                logger.debug('no more tasks to queue')
-                break
-            queued += 1
-        logger.info(f'queued {queued} tasks')
-        self.write({'queued': queued})
-
-
-class TasksActionsProcessingHandler(APIBase):
-    """
-    Handle task action for queued -> processing.
-    """
-    @authorization(roles=['admin', 'system'])
-    async def post(self):
-        """
-        Take one queued task, set its status to processing, and return it.
+        Take one waiting task, set its status to queued, and return it.
 
         Body args (json):
             requirements: dict
@@ -602,9 +696,10 @@ class TasksActionsProcessingHandler(APIBase):
         Returns:
             dict: <task dict>
         """
-        filter_query = {'status':'queued'}
+        filter_query = {'status': 'waiting'}
         sort_by = [('priority',-1)]
         site = 'unknown'
+        queue_instance_id = uuid.uuid1().hex
         if self.request.body:
             data = json.loads(self.request.body)
             # handle requirements
@@ -636,8 +731,8 @@ class TasksActionsProcessingHandler(APIBase):
         print('filter_query', filter_query)
         ret = await self.db.tasks.find_one_and_update(
             filter_query,
-            {'$set':{'status':'processing'}},
-            projection={'_id':False},
+            {'$set': {'status': 'queued', 'site': site, 'instance_id': queue_instance_id}},
+            projection={'_id': False},
             sort=sort_by,
             return_document=pymongo.ReturnDocument.AFTER
         )
@@ -645,71 +740,143 @@ class TasksActionsProcessingHandler(APIBase):
             logger.info('filter_query: %r', filter_query)
             self.send_error(404, reason="Task not found")
         else:
-            self.statsd.incr('site.{}.task_processing'.format(site))
+            self.statsd.incr('site.{}.task_queued'.format(site))
             self.write(ret)
             self.finish()
 
 
-class TasksActionsErrorHandler(APIBase):
+class TasksActionsProcessingHandler(APIBase):
     """
-    Handle task action on error (* -> reset).
+    Handle task action for queued -> processing.
     """
     @authorization(roles=['admin', 'system'])
     async def post(self, task_id):
         """
-        Take one task, set its status to reset.
+        Take one queued task, set its status to processing.
 
         Args:
             task_id (str): task id
 
         Body args (json):
+            instance_id (str): task instance id
+            site (str): (optional) site the task is running at
+
+        Returns:
+            dict: <task dict>
+        """
+        data = json.loads(self.request.body)
+        if (not data) or 'instance_id' not in data:
+            raise tornado.web.HTTPError(400, reason='Missing instance_id in body')
+
+        filter_query = {
+            'task_id': task_id,
+            'status': {'$in': task_prev_statuses('processing')},
+            'instance_id': data['instance_id'],
+        }
+        update_query = {
+            '$set': {
+                'status': 'processing',
+                'status_changed': nowstr(),
+            },
+        }
+
+        if 'site' in data:
+            update_query['$set']['site'] = data['site']
+
+        ret = await self.db.tasks.find_one_and_update(
+            filter_query,
+            update_query,
+            projection={'_id': False}
+        )
+        if not ret:
+            logger.info('filter_query: %r', filter_query)
+            ret = await self.db.tasks.find_one({'task_id': task_id, 'instance_id': data['instance_id']}, projection={'_id': False})
+            if not ret:
+                self.send_error(404, reason="Task not found")
+                return
+            elif ret['status'] != 'processing':
+                self.send_error(400, reason="Bad state transition for status")
+                return
+
+        self.write(ret)
+        self.finish()
+
+
+class TasksActionsErrorHandler(APIBase):
+    """
+    Handle task action on error (* -> waiting).
+    """
+    final_status = 'waiting'
+
+    @authorization(roles=['admin', 'system'])
+    async def post(self, task_id):
+        """
+        Take one task, set its status to waiting.
+
+        Args:
+            task_id (str): task id
+
+        Body args (json):
+            instance_id (str): task instance id
             time_used (int): (optional) time used to run task, in seconds
             resources (dict): (optional) resources used by task
             site (str): (optional) site the task was running at
             reason (str): (optional) reason for error
 
         Returns:
-            dict: {}  empty dict
+            dict: <task dict>
         """
-        filter_query = {'task_id': task_id, 'status': {'$ne': 'complete'}}
+        data = json.loads(self.request.body)
+        if (not data) or 'instance_id' not in data:
+            raise tornado.web.HTTPError(400, reason='Missing instance_id in body')
+
+        filter_query = {
+            'task_id': task_id,
+            'status': {'$in': task_prev_statuses(self.final_status)},
+            'instance_id': data['instance_id'],
+        }
         update_query = defaultdict(dict,{
             '$set': {
-                'status': 'reset',
+                'status': self.final_status,
                 'status_changed': nowstr(),
+                'instance_id': '',
             },
             '$inc': {
                 'failures': 1,
             },
         })
-        if self.request.body:
-            data = json.loads(self.request.body)
-            task = await self.db.tasks.find_one(filter_query)
-            if 'time_used' in data:
-                update_query['$inc']['walltime_err_n'] = 1
-                update_query['$inc']['walltime_err'] = data['time_used']/3600.
-            elif 'resources' in data and 'time' in data['resources']:
-                update_query['$inc']['walltime_err_n'] = 1
-                update_query['$inc']['walltime_err'] = data['resources']['time']
-            for k in ('cpu','memory','disk','time'):
-                if 'resources' in data and k in data['resources']:
-                    try:
-                        new_val = float(data['resources'][k])
-                        old_val = task['requirements'][k] if k in task['requirements'] else Resources.defaults[k]
-                        if k == 'cpu':  # special handling for cpu
-                            if new_val <= 1.1 or new_val > 20:
-                                continue
-                            if new_val < old_val*1.1:
-                                continue
-                            new_val = old_val+1  # increase linearly
-                        else:
-                            new_val *= 1.5  # increase new request by 1.5
-                        if isinstance(Resources.defaults[k], (int, list)):
-                            new_val = math.ceil(new_val)
-                    except Exception:
-                        logger.info('error converting requirement %r',
-                                    data['resources'][k], exc_info=True)
+
+        task = await self.db.tasks.find_one(filter_query)
+        if 'time_used' in data:
+            update_query['$inc']['walltime_err_n'] = 1
+            update_query['$inc']['walltime_err'] = data['time_used']/3600.
+        elif 'resources' in data and 'time' in data['resources']:
+            update_query['$inc']['walltime_err_n'] = 1
+            update_query['$inc']['walltime_err'] = data['resources']['time']
+        for k in ('cpu','memory','disk','time'):
+            if 'resources' in data and k in data['resources']:
+                try:
+                    new_val = float(data['resources'][k])
+                    old_val = task['requirements'][k] if k in task['requirements'] else Resources.defaults[k]
+                    if k == 'cpu':  # special handling for cpu
+                        if new_val <= 1.1 or new_val > 20:
+                            continue
+                        if new_val < old_val*1.1:
+                            continue
+                        new_val = old_val+1  # increase linearly
+                    elif new_val < 0.5:
+                        logger.info('ignoring val below 0.5 for %s: %f', k, new_val)
+                        continue
                     else:
-                        update_query['$max']['requirements.'+k] = new_val
+                        new_val *= 1.5  # increase new request by 1.5
+                    if isinstance(Resources.defaults[k], (int, list)):
+                        new_val = math.ceil(new_val)
+                except Exception:
+                    logger.info('error converting requirement %r',
+                                data['resources'][k], exc_info=True)
+                else:
+                    update_query['$max']['requirements.'+k] = new_val
+
             site = 'unknown'
             if 'site' in data:
                 site = data['site']
@@ -732,7 +899,8 @@ class TasksActionsErrorHandler(APIBase):
                     if text in data['reason']:
                         reason = r
                         break
-                self.statsd.incr('site.{}.task_reset.{}'.format(site, reason))
+                self.statsd.incr('site.{}.task_{}.{}'.format(site, self.final_status, reason))
+
         ret = await self.db.tasks.find_one_and_update(
             filter_query,
             update_query,
@@ -740,10 +908,20 @@ class TasksActionsErrorHandler(APIBase):
         )
         if not ret:
             logger.info('filter_query: %r', filter_query)
-            self.send_error(400, reason="Task not found")
-        else:
-            self.write(ret)
-            self.finish()
+            ret = await self.db.tasks.find_one({'task_id': task_id, 'instance_id': data['instance_id']}, projection={'_id': False})
+            if not ret:
+                self.send_error(404, reason="Task not found")
+                return
+            elif ret['status'] != self.final_status:
+                self.send_error(400, reason="Bad state transition for status")
+                return
+
+        self.write(ret)
+        self.finish()
+
+
+class TasksActionsFailedHandler(TasksActionsErrorHandler):
+    final_status = 'failed'
 
 
 class TasksActionsCompleteHandler(APIBase):
@@ -759,28 +937,38 @@ class TasksActionsCompleteHandler(APIBase):
             task_id (str): task id
 
         Body args (json):
+            instance_id (str): task instance id
             time_used (int): (optional) time used to run task, in seconds
             site (str): (optional) site the task was running at
 
         Returns:
-            dict: {}  empty dict
+            dict: <task dict>
         """
-        filter_query = {'task_id': task_id, 'status': 'processing'}
+        data = json.loads(self.request.body)
+        if (not data) or 'instance_id' not in data:
+            raise tornado.web.HTTPError(400, reason='Missing instance_id in body')
+
+        filter_query = {
+            'task_id': task_id,
+            'status': 'processing',
+            'instance_id': data['instance_id'],
+        }
         update_query = {
             '$set': {
                 'status': 'complete',
                 'status_changed': nowstr(),
+                'instance_id': '',
             },
         }
-        if self.request.body:
-            data = json.loads(self.request.body)
-            if 'time_used' in data:
-                update_query['$set']['walltime'] = data['time_used']/3600.
-            site = 'unknown'
-            if 'site' in data:
-                site = data['site']
-                update_query['$set']['site'] = site
-            self.statsd.incr('site.{}.task_complete'.format(site))
+
+        if 'time_used' in data:
+            update_query['$set']['walltime'] = data['time_used']/3600.
+        site = 'unknown'
+        if 'site' in data:
+            site = data['site']
+            update_query['$set']['site'] = site
+        self.statsd.incr('site.{}.task_complete'.format(site))
+
         ret = await self.db.tasks.find_one_and_update(
             filter_query,
             update_query,
@@ -788,10 +976,16 @@ class TasksActionsCompleteHandler(APIBase):
         )
         if not ret:
             logger.info('filter_query: %r', filter_query)
-            self.send_error(400, reason="Task not found or not processing")
-        else:
-            self.write(ret)
-            self.finish()
+            ret = await self.db.tasks.find_one({'task_id': task_id, 'instance_id': data['instance_id']}, projection={'_id': False})
+            if not ret:
+                self.send_error(404, reason="Task not found")
+                return
+            elif ret['status'] != 'complete':
+                self.send_error(400, reason="Bad state transition for status")
+                return
+
+        self.write(ret)
+        self.finish()
 
 
 class TaskBulkStatusHandler(APIBase):
@@ -817,7 +1011,7 @@ class TaskBulkStatusHandler(APIBase):
         tasks = list(data['tasks'])
         if len(tasks) > 100000:
             raise tornado.web.HTTPError(400, reason='Too many tasks specified (limit: 100k)')
-        if status not in task_statuses:
+        if status not in TASK_STATUS:
             raise tornado.web.HTTPError(400, reason='Bad status')
         query = {
             'task_id': {'$in': tasks},
@@ -825,11 +1019,10 @@ class TaskBulkStatusHandler(APIBase):
         update_data = {
             'status': status,
             'status_changed': nowstr(),
+            'instance_id': '',
         }
-        if status == 'reset':
-            update_data['failures'] = 0
 
-        ret = await self.db.tasks.update_many(query, {'$set':update_data})
+        ret = await self.db.tasks.update_many(query, {'$set': update_data})
         if (not ret) or ret.modified_count < 1:
             self.send_error(404, reason="Tasks not found")
         else:
@@ -862,7 +1055,7 @@ class DatasetTaskBulkStatusHandler(APIBase):
         tasks = list(data['tasks'])
         if len(tasks) > 100000:
             raise tornado.web.HTTPError(400, reason='Too many tasks specified (limit: 100k)')
-        if status not in task_statuses:
+        if status not in TASK_STATUS:
             raise tornado.web.HTTPError(400, reason='Bad status')
         query = {
             'dataset_id': dataset_id,
@@ -871,11 +1064,12 @@ class DatasetTaskBulkStatusHandler(APIBase):
         update_data = {
             'status': status,
             'status_changed': nowstr(),
+            'instance_id': '',
         }
         if status == 'reset':
             update_data['failures'] = 0
 
-        ret = await self.db.tasks.update_many(query, {'$set':update_data})
+        ret = await self.db.tasks.update_many(query, {'$set': update_data})
         if (not ret) or ret.modified_count < 1:
             self.send_error(404, reason="Tasks not found")
         else:
@@ -883,6 +1077,151 @@ class DatasetTaskBulkStatusHandler(APIBase):
             self.finish()
 
 
+class DatasetTaskBulkSuspendHandler(APIBase):
+    """
+    Perform a suspend of all tasks in a dataset.
+    """
+    @authorization(roles=['admin', 'user', 'system'])
+    @attr_auth(arg='dataset_id', role='write')
+    async def post(self, dataset_id):
+        """
+        Set multiple tasks' status to suspended.
+
+        Body may have {'jobs': [<job_id>, <job_id>, ...]}
+        or {'tasks': [<task_id>, <task_id>, ...]}.
+        If it does not, all tasks in a dataset are suspended.
+
+        Args:
+            dataset_id (str): dataset id
+
+        Returns:
+            dict: empty dict
+        """
+        query = {
+            'dataset_id': dataset_id,
+            'status': {'$in': task_prev_statuses('suspended')},
+        }
+        update_data = {
+            'status': 'suspended',
+            'status_changed': nowstr(),
+            'instance_id': '',
+        }
+
+        if self.request.body:
+            data = json.loads(self.request.body)
+            if data and 'jobs' in data:
+                jobs = list(data['jobs'])
+                if len(jobs) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many jobs specified (limit: 100k)')
+                query['job_id'] = {'$in': jobs}
+            elif data and 'tasks' in data:
+                tasks = list(data['tasks'])
+                if len(tasks) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many tasks specified (limit: 100k)')
+                query['task_id'] = {'$in': tasks}
+
+        await self.db.tasks.update_many(query, {'$set': update_data})
+        self.write({})
+        self.finish()
+
+
+class DatasetTaskBulkResetHandler(APIBase):
+    """
+    Perform a reset of all tasks in a dataset.
+    """
+    @authorization(roles=['admin', 'user', 'system'])
+    @attr_auth(arg='dataset_id', role='write')
+    async def post(self, dataset_id):
+        """
+        Set multiple tasks' status back to the starting status.
+
+        Body may have {'jobs': [<job_id>, <job_id>, ...]}
+        or {'tasks': [<task_id>, <task_id>, ...]}.
+        If it does not, all tasks in a dataset are reset.
+
+        Args:
+            dataset_id (str): dataset id
+
+        Returns:
+            dict: empty dict
+        """
+        query = {
+            'dataset_id': dataset_id,
+            'status': {'$in': task_prev_statuses(TASK_STATUS_START)},
+        }
+        update_data = {
+            'status': TASK_STATUS_START,
+            'status_changed': nowstr(),
+            'instance_id': '',
+        }
+
+        if self.request.body:
+            data = json.loads(self.request.body)
+            if data and 'jobs' in data:
+                jobs = list(data['jobs'])
+                if len(jobs) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many jobs specified (limit: 100k)')
+                query['job_id'] = {'$in': jobs}
+            elif data and 'tasks' in data:
+                tasks = list(data['tasks'])
+                if len(tasks) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many tasks specified (limit: 100k)')
+                query['task_id'] = {'$in': tasks}
+
+        await self.db.tasks.update_many(query, {'$set': update_data})
+        self.write({})
+        self.finish()
+
+
+class DatasetTaskBulkHardResetHandler(APIBase):
+    """
+    Perform a hard reset of all tasks in a dataset.
+    """
+    @authorization(roles=['admin', 'user', 'system'])
+    @attr_auth(arg='dataset_id', role='write')
+    async def post(self, dataset_id):
+        """
+        Set multiple tasks' status back to the starting status.
+
+        Body may have {'jobs': [<job_id>, <job_id>, ...]}
+        or {'tasks': [<task_id>, <task_id>, ...]}.
+        If it does not, all tasks in a dataset are hard reset.
+
+        Args:
+            dataset_id (str): dataset id
+
+        Returns:
+            dict: empty dict
+        """
+        query = {
+            'dataset_id': dataset_id,
+        }
+        update_data = {
+            'status': TASK_STATUS_START,
+            'status_changed': nowstr(),
+            'failures': 0,
+            'site': '',
+            'instance_id': '',
+        }
+
+        if self.request.body:
+            data = json.loads(self.request.body)
+            if data and 'jobs' in data:
+                jobs = list(data['jobs'])
+                if len(jobs) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many jobs specified (limit: 100k)')
+                query['job_id'] = {'$in': jobs}
+            elif data and 'tasks' in data:
+                tasks = list(data['tasks'])
+                if len(tasks) > 100000:
+                    raise tornado.web.HTTPError(400, reason='Too many tasks specified (limit: 100k)')
+                query['task_id'] = {'$in': tasks}
+
+        await self.db.tasks.update_many(query, {'$set': update_data})
+        self.write({})
+        self.finish()
+
+
 class DatasetTaskBulkRequirementsHandler(APIBase):
     """
     Update the requirements of multiple tasks at once.
@@ -933,8 +1272,8 @@ class DatasetTaskBulkRequirementsHandler(APIBase):
             'dataset_id': dataset_id,
             'name': name,
         }
-        ret = await self.db.tasks.update_many(query, {'$max':reqs})
-        if (not ret) or ret.modified_count < 1:
+        ret = await self.db.tasks.update_many(query, {'$set':reqs})
+        if (not ret) or ret.matched_count < 1:
             self.send_error(404, reason="Tasks not found")
         else:
             self.write({})
