@@ -21,13 +21,13 @@ class Priority:
     async def _populate_dataset_cache(self):
         args = {
             'keys': 'dataset_id|priority|jobs_submitted|tasks_submitted|group|username',
-            'status': 'processing|truncated',
+            'status': 'processing',
         }
         self.dataset_cache = await self.rest_client.request('GET', '/datasets', args)
         dataset_ids = list(self.dataset_cache)
         args = {
             'keys': 'task_id|dataset_id|job_index|task_index',
-            'status': 'waiting|queued|processing|reset',
+            'status': 'idle|waiting|queued|processing',
         }
         futures = set()
         while dataset_ids:
@@ -153,6 +153,9 @@ class Priority:
             group_prio = await self._get_group_prio(group)
             logger.debug(f'{dataset_id} group_prio: {group_prio}')
 
+        num_dataset_jobs = dataset['jobs_submitted']
+        logger.debug(f'{dataset_id} num_dataset_jobs: {num_dataset_jobs}')
+
         num_all_tasks = await self._get_num_tasks()
         logger.debug(f'{dataset_id} num_all_tasks: {num_all_tasks}')
         num_dataset_tasks = dataset['tasks_submitted']
@@ -170,19 +173,19 @@ class Priority:
         elif max_dataset_prio_group > 0:
             priority -= (1. - dataset_prio / max_dataset_prio_group) / 4.
             logger.info(f'{dataset_id} after dataset group adjustment: {priority}')
-        priority *= user_prio
+        priority *= user_prio**.5
         logger.info(f'{dataset_id} after user adjustment: {priority}')
         priority *= group_prio
         logger.info(f'{dataset_id} after group adjustment: {priority}')
 
         # bias against large datasets
-        factor = (10000. / num_dataset_tasks)**.15 if num_dataset_tasks > 0 else 1.
+        factor = (10000. / num_dataset_jobs)**.05 if num_dataset_jobs > 0 else 1.
         if factor > 1:
             factor = 1.
         priority *= factor
         logger.info(f'{dataset_id} after large dataset adjustment: {priority}')
 
-        priority -= (1. * num_dataset_tasks_avail / num_all_tasks) / 5.
+        priority -= (1. * num_dataset_tasks_avail / num_all_tasks) / 4.
         logger.info(f'{dataset_id} after avail tasks adjustment: {priority}')
 
         if priority < 0.:
@@ -206,7 +209,7 @@ class Priority:
         """
         try:
             dataset = await self._get_dataset(dataset_id)
-        except KeyError:
+        except Exception:
             logger.warning(f'cannot find dataset {dataset_id}', exc_info=True)
             return 0.
         if dataset['tasks_submitted'] < 1:
